package com.jz.sms.chuanglan.service.impl;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.jz.sms.api.BizController;
import com.jz.sms.chuanglan.dto.CLNotifyRequest;
import com.jz.sms.chuanglan.dto.CLSendFixedRequest;
import com.jz.sms.chuanglan.dto.CLSendFixedResponse;
import com.jz.sms.chuanglan.dto.CLTemplateResponse;
import com.jz.sms.chuanglan.service.ChuanLanSmsService;
import com.jz.sms.chuanglan.utils.ChuangLanSmsUtil;
import com.jz.sms.chuanglan.config.ChuangLanSMSConfig;
import com.jz.sms.config.ResponseCode;
import com.jz.sms.dto.*;
import com.jz.sms.repository.domain.SmsTemplateInfo;
import com.jz.sms.service.SmsTemplateService;
import com.jz.sms.util.HttpUtil;
import com.jz.sms.util.Md5Util;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;


@SuppressWarnings("DuplicatedCode")
@Service
public class ChuanLanSmsServiceImpl implements ChuanLanSmsService {

    private static final Logger log = LoggerFactory.getLogger(BizController.class);
    private static final String SUCCESS = "success";
    private static final String ERROR = "error";
    private static final String CODE = "code";
    private static final String DATA = "data";

    @Autowired
    private SmsTemplateService templateService;

    @Override
    public CLTemplateResponse createTemplate(String content, String type) throws Exception {
        log.info("进入创蓝创建短信接口模板的逻辑：{}", content);
        Map<String, Object> requestMap = new HashMap<>();
        String appId;
        String username = ChuangLanSMSConfig.interfaceUser;
        String password = ChuangLanSMSConfig.interfacePassword;
        if ("notification".equals(type)) {
            appId = "49";
        } else if ("marketing".equals(type)) {
            appId = "52";
        } else {
            // TODO 自定义异常,发送警告
            log.error("错误，传入的模板类型未知，类型为：{}", type);
            return new CLTemplateResponse(ResponseCode.SYSTEM_ERROR, "未知的模板类型:" + type, 0);
        }

        String timestamp = String.valueOf(System.currentTimeMillis() / 1000);
        String encrypt = Md5Util.encrypt(new StringBuffer(username).append(password).append(timestamp).toString());
        requestMap.put("username", username);
        requestMap.put("timestamp", timestamp);
        requestMap.put("signature", encrypt);
        requestMap.put("appid", appId);
        requestMap.put("content", content);
        log.info("请求创蓝创建短信模板的请求参数为：{}", requestMap);
        String retStr = HttpUtil.doPost(ChuangLanSMSConfig.addTemplate, requestMap);
        log.info("请求创蓝创建短信模板的返回数据为：{}", retStr);
        JSONObject retJson = JSONObject.parseObject(retStr);
        String status = retJson.getString("status");
        if (SUCCESS.equals(status)) {
            JSONObject data = retJson.getJSONObject("data");
            String id = data.getString("id");
            return new CLTemplateResponse(ResponseCode.SUCCESS, id);
        } else {
            String msg = retJson.getString("msg");
            return new CLTemplateResponse(ResponseCode.UPSTREAM_FAIL.getCode(), msg);
        }
    }

    @Override
    public CLSendFixedResponse sendByFixed(String batchId, String content, List<String> phoneList, String type) throws Exception {
        String account;
        String password;
        if ("notification".equals(type)) {
            account = ChuangLanSMSConfig.notificationAccount;
            password = ChuangLanSMSConfig.notificationPassword;
            log.info("发送短信的模式为：{通知类}");
        } else if ("marketing".equals(type)) {
            account = ChuangLanSMSConfig.marketingAccount;
            password = ChuangLanSMSConfig.marketingPassword;
            log.info("发送短信的模式为：{营销类}");
        } else {
            log.info("发送短信的模式为：{未知类型}");
            throw new Exception("123");
        }

        CLSendFixedRequest request = new CLSendFixedRequest(account, password, content, listToString(phoneList), "true", batchId);

        String requestJson = JSONObject.toJSONString(request);
        log.info("【{}】发送固定内容短信,请求上游数据为：{}", batchId, requestJson);
        String responseStr = ChuangLanSmsUtil.sendSmsByPost(ChuangLanSMSConfig.sendFixed, requestJson);
        log.info("【{}】发送固定内容短信,上游返回数据为：{}", batchId, requestJson);

        if (StringUtils.isBlank(responseStr)) {
            return new CLSendFixedResponse("500", "渠道同步响应为空");
        }
        CLSendFixedResponse response = JSON.parseObject(responseStr, CLSendFixedResponse.class);
        return response;
    }

    private String listToString(List<String> list) {
        String s = list.toString();
        return s;
    }

    @Override
    public CLSendFixedResponse sendByVariable(String batchId, String content, List<String> params, String type) throws Exception {
        String account;
        String password;
        if ("notification".equals(type)) {
            account = ChuangLanSMSConfig.notificationAccount;
            password = ChuangLanSMSConfig.notificationPassword;
            log.info("发送短信的模式为：{通知类}");
        } else if ("marketing".equals(type)) {
            account = ChuangLanSMSConfig.marketingAccount;
            password = ChuangLanSMSConfig.marketingPassword;
            log.info("发送短信的模式为：{营销类}");
        } else {
            log.info("发送短信的模式为：{未知类型}");
            throw new Exception("123");
        }

        CLSendFixedRequest request = new CLSendFixedRequest(account, password, content, listToString(params), "true", batchId);

        String requestJson = JSONObject.toJSONString(request);
        log.info("【{}】发送固定内容短信,请求上游数据为：{}", batchId, requestJson);
        String responseStr = ChuangLanSmsUtil.sendSmsByPost(ChuangLanSMSConfig.sendFixed, requestJson);
        log.info("【{}】发送固定内容短信,上游返回数据为：{}", batchId, requestJson);

        if (StringUtils.isBlank(responseStr)) {
            return new CLSendFixedResponse("500", "渠道同步响应为空");
        }
        CLSendFixedResponse response = JSON.parseObject(responseStr, CLSendFixedResponse.class);
        return response;
    }

    @Override
    public DmHubSendResponse sendBatchByFixed(DmHubSendRequest requestDTO) {
        return null;
    }

    @Override
    public DmHubBatchSendResponse sendBatchByVariable(DmHubBatchSendRequest requestDTO) {
        return null;
    }


    @Override
    public void asyncNotify(CLNotifyRequest notifyInfo) {
        String action = notifyInfo.getAction();
        if (!"messageModel".equals(action)) {
            return;
        }
        String id = notifyInfo.getId();
        String status = notifyInfo.getAuditStatus();
        if (!"1".equals(status)) {

            String msg = notifyInfo.getAuditReason();
            boolean update = templateService.updateForSet("status = abnormal,up_reject_msg = " + msg, new EntityWrapper<SmsTemplateInfo>().eq("up_template_id", id));
            // TODO 发送钉钉告警
            log.error("短信模板审核不通过，原因：{}", msg);
        }
    }
}
